package com.biganto.visual.roompark.presentation.screen.photo.util

import android.content.Context
import android.graphics.SurfaceTexture
import android.util.AttributeSet
import android.view.GestureDetector
import android.view.MotionEvent
import android.view.TextureView
import android.view.View
import androidx.viewpager2.widget.ViewPager2
import kotlinx.android.synthetic.main.photo_page_viewholder.view.*
import timber.log.Timber

/**
 * Created by Vladislav Bogdashkin on 22.12.2019.
 */

class PhotoPreviewSlider @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : TextureView(context, attrs, defStyleAttr)
    ,TextureView.SurfaceTextureListener{

    private var texThread: RendererThread?=null

    private var adapter: PhotosAdapter? = null
    private var vp:ViewPager2? = null
    fun setUpViewPager(viewpager:ViewPager2) {
        this.adapter = viewpager.adapter as PhotosAdapter


        with(viewpager) {
            setPageTransformer { page, position ->
                setParallaxTransformation(page, position)
            }
            registerOnPageChangeCallback(object : ViewPager2.OnPageChangeCallback() {
                var actualPosition = 0
                override fun onPageScrolled(
                    position: Int,
                    positionOffset: Float,
                    positionOffsetPixels: Int
                ) {
                    if (position == actualPosition){
                        processSliding(positionOffset)
                    }
                    else{
                        processSliding(positionOffset-1)
                    }

                    super.onPageScrolled(position, positionOffset, positionOffsetPixels)
                }

                override fun onPageSelected(position: Int) {
                    actualPosition = position
                    texThread?.setIndex(position)
                    super.onPageSelected(position)
                }
            })
        }

        vp = viewpager
    }


    private fun setParallaxTransformation(page: View, position: Float) {

        page.apply {

            when {
                position < -1 -> // [-Infinity,-1)
                    // This page is way off-screen to the left.
                    alpha = 1f
                position < -1 -> // [-1,0){
                {

                }
                position <= 1 -> { // [-1,1]
                    photo_view.translationX = -position * (width / 2)

//                    slider.processSliding(position)
                }
                else -> // (1,+Infinity]
                {

                    // This page is way off-screen to the right.
                    alpha = 1f
                }
            }
        }
    }

    fun processSliding(_position:Float){
        texThread?.setSlidePosition(_position)
    }

    val gestureDetector = GestureDetector(context, object : GestureDetector.SimpleOnGestureListener(){
        override fun onDoubleTap(e: MotionEvent?): Boolean {
            Timber.d( "double tap")
            return true
        }

        override fun onScroll(e1: MotionEvent?, e2: MotionEvent?, distanceX: Float, distanceY: Float): Boolean {
            mIsScrolling=true
            Timber.d( "scroll")
            texThread?.perfomDrag(distanceX)

            return true
        }

        override fun onDown(e: MotionEvent?): Boolean {
            Timber.d( "down")
            return true
        }
    })


    private var mIsScrolling = false

    init {
        surfaceTextureListener=this

        val gestureListener = object : OnTouchListener {

            private var gesture: GestureDetector = gestureDetector

            override fun onTouch(v: View, event: MotionEvent): Boolean {
                Timber.w(" ON TOUCH EVENT!")
                val performTouch = gesture.onTouchEvent(event)
                if (performTouch) return  true

                if (event.action == MotionEvent.ACTION_UP) {
                    if (mIsScrolling) {
                        Timber.d("OnTouchListener --> onTouch ACTION_UP")
                        mIsScrolling = false
                        texThread?.perfomDrag(null,mIsScrolling)
                    }
                }

                if (event.action == MotionEvent.ACTION_BUTTON_PRESS)
                    performClick()

                return false
            }
        }
        setOnTouchListener(gestureListener)
    }


    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        Timber.d("On Size Chagned")
        super.onSizeChanged(w, h, oldw, oldh)
    }

    override fun onSurfaceTextureSizeChanged(surface: SurfaceTexture?, width: Int, height: Int) {
        print(" onSurfaceTextureSizeChanged")
        texThread?.setSize(width,height)
        //Ignored
    }

    override fun onSurfaceTextureUpdated(surface: SurfaceTexture?) {
        print(" onSurfaceTextureUpdated")
        //Ignored
    }

    override fun onSurfaceTextureDestroyed(surface: SurfaceTexture?): Boolean {
        texThread?.isStopped=true
        return true
    }

    override fun onSurfaceTextureAvailable(surface: SurfaceTexture?, width: Int, height: Int) {
        Timber.d("avaliable surf: $surface")
        surface?.let {
            if (texThread?.isAlive==true)
                return
            texThread = RendererThread(context,it)
            texThread?.setSize(width,height)
            texThread?.onChangePage = {ind -> vp?.setCurrentItem(ind,false)}
            texThread?.start()
            adapter?.let {adapter ->
                texThread?.setAdapter(adapter.photosPreviewList)
//                texThread?.setIndex(min(_l.size,2))
            }?: error("adapter not ready!")
        }
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        texThread?.isStopped=true
    }
}


